<?php
/*

 $Id: option.php,v 1.19 2003/11/11 22:45:58 vollmerk Exp $
 $Source: /data/cvsroot/maintain/docs/option.php,v $

 Copyright (c) 2002 Oregon State University - Network Engineering
 All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/*!
	@header DHCP Options Document
	This is a subdocment of workgoups and subnets, it is never referenced on it's own, simply called for functions.
*/

/* Start House Keeping */

	require("../modules/config.php");

	// make sure they are supposed to be here
        if (!check_session()) { logout(); }
	
	setup_user_environment($action, $zone_id);

/* End House Keeping */

// Only admins and zone admins should be here
if ( ! $user->is_admin && !$user->is_zone_admin) {
        header("Location: " . $conf['web_path'] . "/index.php");
	exit();
}

show_template('header');

/*****************
 START WORK BLOCK
*****************/

// Depending on the type create a object
if ($type == 'host') { 
	$object = new Host($type_id);
}
elseif ($type == 'workgroup') { 
	$object = new Workgroup($type_id);
}
elseif ($type == 'subnet') { 
	$object = new Subnet($type_id);
}

$option = new Option($option_id);

/*!
	@function create_option
	@discussion new create option, used the object_option table
		you have to pass a object, it's scope && type
*/
if ($action === 'create_option') {

	// Check and make sure they didn't make a host level subnet etc..
	if (!$option->check_scope($scope,$type)) {
		$conf['error']->add_error('scope',"This Scope now allowed with this DHCP option type");
	}

	$option->dhcp_option = $dhcp_option;
	$option->dhcp_option_name = $option->get_option_name($dhcp_option);

	$option->check_option_value($option->id,$value);

	// Add catching for subnet && not a full admin
	if ($type === 'subnet' && !$user->is_admin) {
		$conf['ui']->show_access_denied();
		exit();
	}
	
	// Add catching for workgroup and less than zone admin
	if ($type === 'workgroup' && $user->has_access($conf['zone']->id) < 50) {
		$conf['ui']->show_access_denied();
		exit();
	}

	if ($type === 'host' && !$user->has_access($object->zone->id)) {
		$conf['ui']->show_access_denied();
		exit();
	}

	if (!$conf['error']->error_state) {

		$option_id = $option->create_option($type_id,$type,$scope,$dhcp_option,$value);
		$option = new Option($option_id);

		$conf['ui']->display_confirmation("DHCP Option created", "The $type DHCP option has been created", $conf['web_path'] . "/option.php?action=show_options&amp;object_id=$type_id&amp;type=$type");

	} // if no error

	else {
		$action = 'show_create_option';
	} //an error occured

} // create_option

/*!
	@function update_option
	@discussion updated the dhcp option this can
		update any type of option
*/
elseif ($action == 'update_option') {

	if ($user->has_access($zone_id) < 50 && !$user->id_admin) {
		$conf['ui']->show_access_denied();
		exit();
	}
        
	// Check and make sure they didn't make a host level subnet etc..
        if (!$option->check_scope($scope,$type)) {
                $conf['error']->add_error('scope',"This Scope now allowed with this DHCP option type");
        }
	
	$option->check_option_value($dhcp_option,$value);
	

	if (!$conf['error']->error_state) {

		$option->update_value($value);
		$option->update_option($dhcp_option);
		$option->update_scope($scope);

		$conf['ui']->display_confirmation("DHCP Option Updated", "Your DHCP option has been updated", $conf['web_path'] . "/option.php?action=show_options&object_id=$object_id&type=$type");

	} // no errors

	else {
		$action = 'show_edit';
	}

} // update_option


// now do things based on the form data
elseif ( $action == 'Create' || $action == 'Update' ) {

	if (!is_object($option)) { 
		$option = new Option(0);
	}
	$option->check_option_value($dhcp_option_id,$value);

	if ( $conf['error']->error_state ) {
		// there was an error so set the appropriate action
		if ( $action == 'Create' ) {
			$action = 'show_create';
		}
		else {
			$action = 'edit';
			$option = new Option($option_id, $type);
		}

		$option->id                = $option_id;
		$option->value		   = $value;
	}
	else {
		// create/update the stinkin' option
		if ( $action == 'Create' ) {

			if ($type != 'workgroup') { 
				$sql = "INSERT INTO $type"."_option_local (dhcp_option,value,$type) ".
					"VALUES ($dhcp_option_id,'$value',$type_id)";
			}
			else {
				if (!$option_type) { $option_type = 'group'; }
				$sql = "INSERT INTO workgroup_option_local (dhcp_option,value,workgroup,type) " . 
					"VALUES ($dhcp_option_id,'$value','$type_id','$option_type')";
			}
			$db_result = mysql_query($sql, $conf['dbh']);	
			$option_id  = mysql_insert_id($conf['dbh']);
		}
		else {
			if ($type != 'workgroup') { 
				$sql = "UPDATE $type"."_option_local SET dhcp_option=$dhcp_option_id,value='$value',$type=$type_id".
				" WHERE id=$option_id";
			}
			else {
				if (!$option_type) { $option_type = 'group'; }
				$sql = "UPDATE workgroup_option_local SET dhcp_option=$dhcp_option_id,value='$value',workgroup='$type_id',type='$option_type' WHERE id='$option_id'";
			}
			
			$db_result = mysql_query($sql, $conf['dbh']);	
		}

		$action = 'show_options';
		$option = new Option($option_id, $type);
	}
}

/*!
	@function Update_Host_Option
	@discussion this updates the host DHCP options... 
*/
elseif ($action == 'Update Host Option') {

	$host = new Host($option->type_id);

	if ($user->has_access($host->zone->id) >= 50) { 

	        $option->check_option_value($dhcp_option_id,$value);
		
		if (!$conf['error']->error_state) {
			$option->update_host_option($dhcp_option_id,$value,$option_id);
			$conf['ui']->display_confirmation("Option Updated", "Host Option set to $value", $conf['web_path'] . "/host.php?action=show_host_options&host_id=" . $host->id);
		} // if no errors
		else {
			$action = 'show_edit_host';
		}
	}
	else {
		$conf['ui']->show_access_denied();
	}


} // update_host_option

// this we're really deleting this zone
elseif ( $action == 'delete') {

	if (!$user->is_admin && $user->has_access($zone_id) < 50) {
		$conf['ui']->show_access_denied();
		exit();
	}

	$option->delete();

	$conf['ui']->display_confirmation("DHCP Option deleted","The dhcp option has been deleted", $conf['web_path'] . "/option.php?action=show_options&object_id=$object_id&type=$type");

	
} // if we are deleting

/*!
	@function create_host
	@discussion creates a host specifiec dhcp option
*/
elseif ($action == 'create_host') { 
	// Create the host object we need
	$host = new Host($type_id);
	$option = new Option();
	
	if ( !$user->is_admin && $user->has_access($host->zone->id) < 50 ) { 
		$conf['ui']->show_access_denied(); 
	} // if they don't have access

	// if they do have access
	else {

		$option->check_option_value($dhcp_option_id,$value);

		if (!$conf['error']->error_state) { 
			$option->create_host_option($dhcp_option_id,$value,$type_id);
			$conf['ui']->display_confirmation("Host DHCP option created", "Your Host DHCP option has been added", $conf['web_path'] . "/host.php?action=show_host_options&host_id=$type_id");
		}
		else {
			$action = 'show_create_host';
		}

	} // end if they have access

} // create_host

/*!
	@function create_global
        @discussion creates the global workgroup option
*/
elseif ($action == 'create_global' AND $user->is_admin) {

	// Create the needed objects
        $option = new Option();

	$option->check_option_value($dhcp_option_id,$value);

	if (!$conf['error']->error_state) { 
	        $option->create_global_option($dhcp_option_id,$value, $type);
	        $conf['ui']->display_confirmation("Global $type option created", "Your global $type option has been added", $conf['web_path'] . "/admin.php");
	}
	else {
		$action = 'show_create_global';
	}

} // create_global

/*!
	@function global_delete
	@discussion deletes a global option
*/
elseif ($action == 'global_delete' && $user->is_admin) {

	$option->delete_global_option($option_id,$type);
	$conf['ui']->display_confirmation("Global $type option delete", "Your global $type option has been deleted", $conf['web_path'] . "/admin.php");

} // global_delete

/*!
	@function really_delete_host
	@discussion this is the actual function that deletes a host_option
*/
elseif ($action == 'really_delete_host' && $user->has_access($zone_id) >= 50) {

	$option->delete_host_option($option_id);
	$conf['ui']->display_confirmation("Option Deleted", "The host option has been removed.", $conf['web_path'] . "/host.php?action=show_host_options&host_id=$host_id");

} // really_delete_host

/****************************
*****************************
     START DISPLAY BLOCK
*****************************
*****************************/

?>
<font class="header1"><?= $title; ?></font>
<?

/*!
	@function confirm_delete
	@discussion confirm that you really want to delete the 
		selected dhcp option
*/
if ($action == 'confirm_delete') {

	$conf['ui']->show_confirm($conf['web_path'] . "/option.php?action=delete&type=$type&type_id=$type_id&next_page=$next_page&next_action=$next_action&zone_id=$zone_id", "Are you sure you want to delete this DHCP option");


} // confirm_delete

/*!
	@function show_edit
	@discussion display a page to edit a single option on
		we will need to re-pass the next_page && the 
		next_action
*/
elseif ($action == 'show_edit') {
	if (!$user->has_access($zone_id) < 50 && !$user->is_admin) {
		$conf['ui']->show_access_denied();
		exit();
	}
	require ($conf['prefix'] . "/templates/show_option.inc");

} // show_edit

	elseif ( $action == 'edit' || $action == 'show_create' ) {
		// show the range edit form
		$conf['ui']->show_option($action, $type, $type_id, $option);
	}

	/*!
		@function edit_host_option
		@discussion edits a host DHCP option
	*/
	elseif ($action == 'show_edit_host') { 
		$option = new Option($option_id,$type);
		require ($conf['prefix'] . "/templates/show_host_option.inc");
	} // show_host_accountant

	/*!
		@function show_create_host
		@discussion show the host create template
	*/
	elseif ($action == 'show_create_host') { 
		require ($conf['prefix'] . "/templates/show_host_option.inc");
	}

	/*!
		@function show_options
		@dicussion displays many options, based on type
		@param type	The type of object we need to show the options for
		@param type_id	The ID of the object we need to show
		@param page	The final page we go to if we need to get back to this object
		@param next_action The final action we need to do to get back to this object 
	*/
	elseif ( $action == 'show_options' ) {
		$options = $conf['ui']->get_options("SELECT id FROM object_option WHERE type='$type' AND object_id='$object_id'");	
		$conf['ui']->show_options($action, $type, $object_id, $options);
	 } // show_options

	/*!
		@function show_global_options
		@discussion show global options.. duh
	*/
	elseif ($action == 'show_global_options') {
		global $conf;

		$sql = "SELECT id FROM $type"."_option_global";
		$db_results = mysql_query($sql, $conf['dbh']);

		$options = array();

		while ($r = mysql_fetch_object($db_results)) {
			$options[] = new Option($r->id,$type,"true");
		}
		$sql = "SELECT id FROM option WHERE $type=$type_id";
	
		if (!$user->view) {
	                $user->view = new View($sql);
	        }

		if ($sort_type) {
			$user->view->initialize($sort_type,$sort_offset);
	        }
	        else {
			$user->view->initialize('dhcp_option', $sort_offset);
	        }
	
		$conf['ui']->show_options($action, $type,$type_id,$options);

	} // show_global_options

	/*!
		@function Delete_Global
		@discussion Delete a global option of $type
	*/
	elseif ($action == 'Delete Global' || $action =='delete_global' ) {
		$option = new Option($option_id,$type,"true");
		$conf['ui']->show_confirm($conf['web_path'] . "/option.php?action=global_delete&option_id=$option_id&type=$type", "Are you sure you want to delete this DHCP option? ($option->dhcp_option_name with value $option->value)");


	} // delete_global

	/*!
		@function Delete_Host
		@discussion Shows a confirmation for deleting a host dhcp optoin
	*/
	elseif ($action == 'Delete Host Option' || $action == 'delete_host' ) {

		$conf['ui']->show_confirm($conf['web_path'] . "/option.php?action=really_delete_host&option_id=$option_id&zone_id=$zone_id&type=host&host_id=$host_id", "Are you sure you want to delete this option?");		

	} // delete_host	
        /*!
                @function show_create_global
                @discussion shows you the create global workgroup options
        */
        elseif ($action == 'show_create_global') {
                require($conf['prefix'] . "/templates/show_global_option.inc");
        }

	/*!
		@function show_create_option
		@discussion display the create page for a new
			dhcp option, this is a universal page for all options
	*/
	elseif ($action === 'show_create_option') {
		require($conf['prefix'] . "/templates/show_create_option.inc");
	}

?>

<? show_template('footer'); ?>
