<?php
/*

 $Id: host.php,v 1.87 2003/11/25 19:12:50 vollmerk Exp $
 $Source: /data/cvsroot/maintain/modules/class/host.php,v $

 Copyright (c) 2002 Oregon State University - Network Engineering
 All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/*!
	@header Host Class
	This is the base host class ... it stores all of the information
	needed to update/modify/view a host.

	There is a little trick to the IP stuff on our hosts now. There
	are two fields $ip and $dynamic_range. If the host is a dynamic host
	then IP is 0 and dynamic_range is... the dynamic range. If it is 
	a static host then dynamic_range is 0 and IP is the int value of 
	the ip address. 

*/

class Host {

	//Var's gathered from the host table
	var $id;
	var $ip;
	var $name;
	var $ha;
	var $os;
	var $type;
	var $location;
	var $serial;
	var $other_id;
	var $last_seen;
	var $status;
        var $last_update;
        var $who_update;
	var $bandwidth_rate; // enum (full,limited,blocked)
	var $expire; // for date deletion
	var $ttl; // Time to live for the host

	//These are ID's which generate objects seperatly
	var $dynamic_range;
	var $domain;
	var $workgroup;
	var $zone;
	var $full_ha;

	// this is the name + domain name (constructed)
	var $hostname;

	// the object history information
	var $object_history;

	// we can populate this stuff from the database -- no sense in
	//   passing it in.
	function Host ($host_id) {
		global $conf;

		$this->id = $host_id;

		if ($host_id) {
			$info = $this->get_info();
		}

		if ( ! $info ) {
			return false;
		}

		//things found in the host table
		$this->name	      = $info->name;
		$this->ha	      = $info->ha;
		$this->os	      = $info->os;
		$this->type	      = $info->type;
		$this->location	      = $info->location;
		$this->serial	      = $info->serial;
		$this->other_id	      = $info->other_id;
		$this->status         = $info->status;
		$this->bandwidth_rate = $info->bandwidth_rate;
		$this->purchase_date  = $info->purchase_date;
		$this->po_number      = $info->po_number;
		$this->warranty_date  = $info->warranty_date;
		$this->owning_unit    = $info->owning_unit;
		$this->user_id        = $info->user_id;
		$this->last_seen      = $info->last_seen;
                $this->last_update    = $info->last_update;
                $this->who_update     = $info->who_update;
		$this->expire         = $info->expire;
		$this->ttl	      = $info->ttl;

		//Objects (requires more sql calls)
		$this->workgroup = new Workgroup($info->workgroup);
		$this->domain	 = new Domain($info->domain);
		$this->zone	 = new Zone($info->zone);

		// build the object_history object
		$this->object_history = new ObjectHistory('host', $this->id);

		//buid the full ha with :'s and everything!
		$built_ha = $info->ha{0} . $info->ha{1} . ":" . $info->ha{2} . $info->ha{3} . ":" . $info->ha{4} . $info->ha{5} . ":" . 
			    $info->ha{6} . $info->ha{7} . ":" . $info->ha{8} . $info->ha{9} . ":" . $info->ha{10} . $info->ha{11};
		$this->full_ha = $built_ha;
	
		// dynamic IP's are set to 0
		if ( $info->ip ) {
			$this->ip	= new IP($info->ip);
		}
		else {
			$this->dynamic_range = new Range($info->dynamic_range);
			$this->ip 	     = 0;
		}

		// do this last because we need Domain dummy
		$this->hostname = $this->name . "." . $this->domain->name;

	} // Host

	function get_info () {
		global $conf;

		$sql = "SELECT * FROM host WHERE id='$this->id'";
		$db_results = mysql_query($sql, $conf['dbh']);

		return mysql_fetch_object($db_results);
	} // get_info

	/*
		@function get_workgroups
		@discussion this runs through a little to figure out which workgroups this host has access to 
			returns in an array
	*/
	function get_workgroups($zone_id =0) {
		global $conf;

		// Next we need to check the workgroup mappings
		if (!$zone_id) { $zone_id = $this->zone->id; }
		if (!$zone_id) { $zone_id = $conf['zone']->id; }
		$sql = "SELECT workgroup FROM zone_workgroup WHERE zone_workgroup.zone='$zone_id'";
		$db_results = mysql_query($sql, $conf['dbh']);

		while ($r = mysql_fetch_object($db_results)) {
			$workgroup = new Workgroup($r->workgroup);
			$results[$workgroup->name] = $workgroup;
		} // end while
		
		$default_group = $conf['ui']->get_workgroups("SELECT * FROM workgroup WHERE name LIKE '" . $conf['default_workgroup'] . "'");
		$results = array_merge($default_group,$results);
		
		return $results;

	} // get_workgroups

	//get the range for this host
	function get_range() {
		global $conf;
	
		$sql = "SELECT * FROM ranges WHERE start<=" . $this->start->ip_int . " AND end>=" . $this->end->ip_int;

		$db_results = mysql_query($sql, $conf['dbh']);

		$r = mysql_fetch_object($db_results);
		$results = new Range($r->id);

		return $results;

	} //get_range

	// get_host_history
	function get_host_history($ip_int=0,$ha=0) {
		global $conf;

		// Check to see if we are using a real object
		// This is bassed of $this->ha
		if ($this->id) { 
			$sql = "SELECT DISTINCT(date),ip,ha FROM host_history WHERE ha='$this->ha' ORDER BY date DESC";
		}
		//If they passed a ip_int base it on that
		elseif ($ip_int) {
			$sql = "SELECT DISTINCT(date),ip,ha FROM host_history WHERE ip='$ip_int' ORDER BY date DESC";
		}
		//lastly go for the ha
		else {
			$sql = "SELECT DISTINCT(date),ip,ha FROM host_history WHERE ha='$ha' ORDER BY date DESC";
		}

		$db_results = mysql_query($sql, $conf['dbh']);

		while ($r = mysql_fetch_object($db_results)) {
			$results[] = $r;
		} //end while
	
	return $results;

	} // get_host_history

	/*!
		@function get_options
		@discussion gets the host specific workgroup options
	*/
	function get_options() {
		global $conf;

		$sql = "SELECT id FROM object_option WHERE object_id='$this->id' AND type='host'";
		$db_results = mysql_query($sql, $conf['dbh']);

		while ($r = mysql_fetch_array($db_results)) {
			$results[] = new Option($r['id']);
		}

		return $results;

	} // get_options

	/*!
		@function get_history_ips
		@discussion get a listing of all of the ips this host
			has had according to the host history
	*/
	function get_history_ips() {

		$last_date = 0;

		$sql = "SELECT * FROM host_history WHERE ha='$this->ha' ORDER BY date ASC";
		$db_results = mysql_query($sql, dbh());

		while ($history = mysql_fetch_object($db_results)) {
		
			$data->start = $last_date;
			$data->int = $history->ip;
			$data->end = $history->date;
			$last_date = $history->date;
			$results[] = $data;

		} // while results

		return $results;

	} // end get_history_ips

	// check to see if $this has host_history
	function has_host_history() {
		global $conf;

		$sql = "SELECT id FROM host_history WHERE ha='$this->ha' LIMIT 1";
		$db_results = mysql_query($sql, $conf['dbh']);
		
		$r = @mysql_fetch_object($db_results);

		return $r;

	} //has_host_history

	// massage the last_seen date for this host
	function show_last_seen() {
		if ( $this->last_seen ) {
			echo date("F j, Y, g:i a", $this->last_seen);
		}
		else {
			echo "N/A";
		}
	}

	/*!
		@function show_expire_date
		@discussion Shows the expire date for the host
	*/
	function show_expire_date() {
		if ($this->expire) {
			echo date("F j, Y, g:i a", $this->expire);
		}
		else {
			echo "Never Expires";
		}
	} // show_expire_date

	function update_purchase_date($purchase_date) {
		$this->update_item('purchase_date', $purchase_date);
		$this->purchase_date = $purchase_date;
	}

	function update_zone($zone_id) {
		$this->update_item('zone',$zone_id);
		$this->zone = new Zone ($zone_id);
	}

	function update_po_number($po_number) {
		$this->update_item('po_number',$po_number);
		$this->po_number = $po_number;
	} 

	function update_warranty_date($warranty_date) {
		$this->update_item('warranty_date',$warranty_date);
		$this->warranty_date = $warranty_date;
	}

	function update_owning_unit($owning_unit) {
		$this->update_item('owning_unit',$owning_unit);
		$this->owning_unit = $owning_unit;
	}

	function update_user_id($user_id) {
		$this->update_item('user_id',$user_id);
		$this->user_id = $user_id;
	}

	function update_name ($newname) {
		$this->update_item('name', $newname);
		$this->name = $newname;
	}

	function update_status ($new_status) {
		$this->update_item('status',$new_status);
		$this->status = $new_status;
	}

	function update_bandwidth_rate ($new_rate) {
		$this->update_item('bandwidth_rate', $new_rate);
		$this->bandwidth_rate = $new_rate;
	} 

	function update_expire ($new_expire) { 
		$this->update_item('expire',$new_expire);
		$this->expire = $new_expire;
	}

	function update_ttl($new_ttl) {
		$this->update_item('ttl',$new_ttl);
		$this->ttl = $new_ttl;
	}

	function update_workgroup ($new_workgroup) {
		$this->update_item('workgroup',$new_workgroup); 
		$this->workgroup = $new_workgroup;
	}

	function update_domain ($new_domain) {
		$this->update_item('domain', $new_domain);
		$this->domain = new Domain($new_domain);
	}

	function update_ha ( $new_ha ) {
		$this->update_item('ha', $new_ha);
		$this->ha = $new_ha;
	}

	function update_type ( $new_type ) {
		$this->update_item('type', $new_type);
		$this->type = $new_type;
	}

	function update_os ( $new_os ) {
		$this->update_item('os', $new_os);
		$this->os = $new_os;
	}

	function update_location ( $new_location) {
		$this->update_item('location', $new_location);
		$this->location = $new_location;
	}

	function update_serial ( $new_serial ) {
		$this->update_item('serial', $new_serial);
		$this->serial = $new_serial;
	}

	function update_other_id ( $new_other_id ) {
		$this->update_item('other_id', $new_other_id);
		$this->other_id = $new_other_id;
	}


	//this updates the dynamic_range and ip address at the same time! Woo!
	function update_ip ( $new_ip=0,$new_range=0 ) {
		global $conf; 

		if ($new_ip) {
			$this->update_item('ip', $new_ip);
			$this->ip = $new_ip;	
			
			//just incase let's kill the dynamic_range entry
			$this->update_item('dynamic_range',"0");	
			$this->dynamic_range = "0";		
		}
		elseif ($new_range) {
			$this->update_item('dynamic_range',$new_range);
			$this->dynamic_range = $new_range;

			//just incase let's kill the ip entry
			$this->update_item('ip',"0");
			$this->ip="0";
		}

	} //update_ip

	function update_item ($type, $new_item) {
		global $user;

		// do a last min check to make sure they have access
		if ($user->has_access($this->zone->id)) {

			$sql = "UPDATE host SET $type='$new_item' WHERE id='$this->id'";
			$db_result = mysql_query($sql, dbh());
		}

	} //update_item

        /*!
		@function is_active
		@discussion check to see if this host is active (basicly has it been $conf['update_time'] 
			    since we last updated the host
	*/
        function is_active() {
                global $conf;

		$history = $this->object_history->get_history();

                if ($history->date + $conf['update_time'] <= time()) {
                        return true;
                }

	        return false;

        } //is_active

	/*!
		@function delete
		@discussion deletes a host and all the little records 
			assoicated with it, includes any options then we need to 
			add a deletion record
	*/
	function delete() {
		global $conf,$user;
		
		// Delete the host!
		$sql = "DELETE FROM host WHERE id='$this->id'";
		$db_result = mysql_query($sql, dbh());

		// Delete any options..
		$sql = "DELETE FROM object_option WHERE type='host' AND object_id='$this->id'";
		$db_results = mysql_query($sql, dbh());

		// Delete any object_history
		$sql = "DELETE FROM object_history WHERE type='host' AND object_id='$this->id'";
		$db_results = mysql_query($sql, dbh());

		// Add the deletion record
		$text = "Deleted Host " . $this->hostname . "(" . $this->full_ha . ")";
		if ($this->ip->ip_int) {
			$text .= " IP:" . $this->ip->ip_addr;
		}
		else {
			$text .= " Dynamic Address:" . $this->dynamic_range->start->ip_addr;
		}
		$conf['ui']->add_delete_history($this->zone->id, $text, $user->username);

	} //delete

	//get batch of hosts
	// this is used by bulk delete/update and show hosts this function takes the imput and returns 
	// an array of hosts. 
	function bulk_get($select_type,$range_id=0,$start_range=0,$end_range=0,$domain_id=0,$hostname=0,$select_field=0,$value=0) {
		global $conf;

                //ok we've got to get the hosts for viewing
                //check to see what kind of select they did and grab the
                //correct hosts

                //zone -> range grab
                if ($select_type == "defined_range") {
                        $range = new Range($range_id);
                        $hosts = $range->get_range_hosts();
                }
                //custom range grab
                elseif ($select_type == "custom_range") {
                //check to make sure range was entered correctly
                if (! preg_match("/^\d+\.\d+\.\d+\.\d+$/", $start_range) ) {
                        $conf['error']->add_error('start_range', "Invalid Start Range");
                        }
                if (! preg_match("/^\d+\.\d+\.\d+\.\d+$/", $end_range) ) {
                        $conf['error']->add_error('end_range', "Invalid End Range");
                        }
                $ip = new Ip();
                $start_int = $ip->ip2int($start_range);
                $end_int = $ip->ip2int($end_range);
                if ($start_int >= $end_int) {
                        $conf['error']->add_error('end_range', "End range is before start range");
                        }
                if (!$conf['error']->error_state) {
                        $zone_id = $conf['zone']->id;
                        $hosts = $conf['ui']->get_hosts("SELECT id FROM host WHERE ip<='$end_int' AND ip>='$start_int' AND zone='$zone_id' ORDER BY ip DESC");

                } //check for error state

                } //custom range grab

                //zone -> domain grab
                elseif ($select_type == "domain") {
                        $domain = new Domain($domain_id);
                        $hosts = $domain->get_hosts();
                }
                //fuzzy host grab
                elseif ($select_type == "hostname") {
                        if (substr($hostname,strlen($hostname)-1,1) != "%") { $hostname = $hostname ."%"; }
                        if (substr($hostname,0,1) != "%") { $hostname = "%" . $hostname; }

                        $zone_id = $conf['zone']->id;
                        $hosts = $conf['ui']->get_hosts("SELECT * FROM host WHERE name LIKE '$hostname' AND zone='$zone_id'");
                }

		elseif ($select_type == 'other') {
			$zone_id = $conf['zone']->id;
			$value = "%" . $value . "%";
			$sql = "SELECT id FROM host WHERE $select_field LIKE '$value' AND zone='$zone_id'";
			$hosts = $conf['ui']->get_hosts($sql);
		}
		
		return $hosts;

	} // bulk_get

	/*!
		@function check_workgroup
		@discussion checks the workgroup, and if they are not a admin, it makes 
			sure that the workgroup is allowed within their zone
	*/
	function check_workgroup($workgroup, $zone_id=0) {
	
		if ($user->is_admin) { return true; }
		if (!$zone_id) { $zone_id = $this->zone->id; }
	
		return $workgroup->has_zone($zone_id);
	

	} // check_workgroup


	/*!
		@function check_hostname
		@discussion the hostname (not fully-qualified) and a Domain object
	*/
	function check_hostname($name, $domain, $host_id = 0) {
		global $conf;

		if (strlen($name) < 1) { 
			$conf['error']->add_error('name',"No name entered. Please enter a name for this host.");
		}

		// because we format our dynamic hosts 128-193-188-118.yoursite.com we can't let people
		// create hosts like that normaly
		if ( preg_match ("/^\d{1,3}\-\d{1,3}\-\d{1,3}\-\d{1,3}$/",$name)) {
			$conf['error']->add_error('name', "$name not allowed, it would overlap with auto-generated dynamic hosts");
		}

		if ( preg_match ("/[^\-A-Za-z0-9]+/", $name) ) {
			$conf['error']->add_error('name', "$name.$domain->name is an invalid hostname.");	
			return false;	
		}
		
		// we have to use the host_id to determine if they working on that host currently
		//   this is used during the update of a single host
		// we do a like here in so we get the case-insensitive match
		$sql = "SELECT * FROM host WHERE name LIKE '$name' AND domain='$domain->id'";
		if ($host_id) { $sql .= " AND id!='$host_id'"; }
		$db_result = mysql_query($sql, $conf['dbh']);
		$host_info = mysql_fetch_object($db_result);
		if ( $host_info ) {
			if ( $host_info->id != $host_id ) {
				$link = "<a href=\"" . $conf['web_path'] . "/host.php?action=show_host&host_id=" . $host_info->id . "\">[ Show Host ]</a>";
				$conf['error']->add_error('name', "$name.$domain->name is already in use by a Host. " . $link);	
				}
		} // if host_info


		// now we need to make sure there isn't a forward pointer with this name.. 
		$sql = "SELECT id FROM pointer WHERE hostname LIKE '" . $name . $domain->name . "'";
		$db_results = mysql_query($sql, $conf['dbh']);
		$host_info = mysql_fetch_row($db_results);
	
		if ($host_info) {
			$link = "<a href=\"" . $conf['web_path'] . "/pointer.php?action=show_pointer&pointer_id=" . $host_info[0] . "\">[ Show Pointer ]</a>";
			$conf['error']->add_error('name', "$name.$domain->name is already in use by a Pointer. " . $link);
		}

		// now we need to check to make sure there isn't a Cname with this name.
		$sql = "SELECT id FROM zone_cname WHERE name LIKE '$name' AND domain='$domain->id'";
		$db_results = mysql_query($sql, $conf['dbh']);
		$host_info = mysql_fetch_row($db_results);
		
		if ($host_info) {
			$link = "<a href=\"" . $conf['web_path'] . "/cname.php?action=show_cname&cname_id=" . $host_info[0] . "\">[ Show Cname ]</a>";
			$conf['error']->add_error('name', "$name.$domain->name is already in use by a CNAME. " . $link);
		}

	} //check_hostname

	//check to make sure it's a valid mac addie
	//in order to do this right we have to supply a range_id if we don't 
	//supply a host object
	function check_hardware_address($ha, $host_id = 0, $range_id,$subnet_id=0) {
		global $conf;

		//for range testing
		$range = new Range($range_id);

		// is it 12 chr's long
		if ( strlen($ha) != 12 ) {
			$conf['error']->add_error('ha', "Hardware address must be exactly 12 characters long.");	
		}

		// does it have valid chrs
		if ( preg_match("/[^0-9a-fA-F]+/", $ha) ) {
			$conf['error']->add_error('ha',
				"Hardware address can only contain hexidecimal numbers (0-9 and A-F).");	
		}

	
		//check for duplicate ha in same zone
		if ($range->type == 'dynamic' OR $range->type == 'ddns') {
			$sql = "SELECT host.id FROM host WHERE ha='$ha' AND dynamic_range='$range_id'";
		}
		else {
			$sql = "SELECT host.id FROM host,ranges WHERE ranges.subnet='".$subnet_id."' AND host.ip<=ranges.end AND host.ip>=ranges.start  AND host.ha='". $ha."' ";
		}
		
		// this is for the case when we are actually updating a host
		if ( $host_id ) {
			$sql .= "AND host.id!='$host_id'";
		}

		$db_result = mysql_query($sql, $conf['dbh']);
		$host_info = mysql_fetch_object($db_result);
	
		if ( $host_info ) {
			$conf['error']->add_error('ha', "The hardware address $ha is already in use.");	
		}
		
	} //check_hostname

	//check_ip_range this checks to see if there are any free addresses in a range
	function check_ip_range($range_id) { 
		global $conf;

		$range = new Range($range_id);
		if (! ( $range->get_free_address_count() ) ) {
			$conf['error']->add_error('ip_range', "There are no free IP addresses in this range.");
		}

	} //check_ip_range


	// For $this host and this zone return the
	//   next host object for this zone.  If there isn't
	//   another host then return nothing.
	function get_next_host_by_name () {
		global $conf;

		$zone_id = $this->zone->id;
		$sql = "SELECT id FROM host WHERE zone=$zone_id ORDER BY name";
		$db_result = mysql_query($sql, $conf['dbh']);

		while ( $host = mysql_fetch_array($db_result) ) {
			if ( $host[0] == $this->id ) {
				$host = mysql_fetch_array($db_result);
				return $host[0];
			}
		}

		return;
	} // get_next_host_by_name


	// For $this host and this zone return the
	//   next host object for this zone by IP address.
	function get_next_host_by_ip () {
		global $conf;

		//if this is a static host then simply get the next possible ip address (get's next in line...)
		if ($this->ip->ip_int) {
			// increment the IP addr and make new object
			$ip_int = $this->ip->ip_int;
			$ip_int++;
	
			$zone_id = $this->zone->id;
			//All we have to do is order by ip on the host table
			//let's keep it in the same zone though
			$sql = "SELECT * FROM host WHERE ip>='$ip_int' AND zone='$zone_id' LIMIT 1";
		}
		//if this is a dynamic host then get the next dynamic host by id
		elseif ($this->dynamic_range->id) {
			//we are dealing with a dynamic address.... 
			$sql = "SELECT * FROM host WHERE dynamic_range='$this->dynamic_range->id' AND zone='$this->zone->id' AND id!='$this->id' ORDER BY id LIMIT 1";
		}

		$db_results = mysql_query($sql, $conf['dbh']);
		$results = mysql_fetch_row($db_results);

		return $results[0];
	} //get_next_host_by_ip


	//this function shows the workgroups that are possible for this host
	//to access... This is used in show_host
	function show_workgroup_select() { 
		global $conf;

		$workgroups = $this->get_workgroups();
                // if its > 1 then show the show_workgroup_select tool
                if ( count($workgroups) > 1 ) {
                        $conf['ui']->show_workgroup_select($workgroups, $this->workgroup->id);
                }
                // else just put a read-only version
                else {
			$workgroup = array_pop($workgroups);
                        $workgroup_name = $workgroup->name;
                        print "<input type=\"hidden\" name=\"workgroup_id\" value=\"" . $workgroup->id . "\" />$workgroup_name";
                }

	} //show_workgroup_select

        /*!
                @function send_notify
                @discussion this prepares --> sends a notification of 
                        a admin host update to all parties who own it
                        works off of $this object
        */
        function send_notify($zones=0,$comment=0) {
                global $conf,$user;

        if (!$zones) { $zones = $this->zone; }

        if (!$conf['send_notifications']) { return false; }	

        $text = $this->hostname . " updated by an Administrator\n\n" .
		"Extra Comment: $comment\n" .
                "HW:$this->ha\nDomain:" . $this->domain->name . "\nOS:$this->os\nType:$this->type\n" . 
                "Location:$this->location\nSerial:$this->serial\nOther_id:$this->other_id\nStatus:$this->status\n" .
                "Purchase Date:$this->purchase_date\nPO Number:$this->po_number\nWarranty Date:$this->warranty_date\n" .
                "Owning Unit:$this->owning_unit\nUser ID:$this->user_id\nTTL:$this->ttl\n\nUpdated by $user->username";
        
        $conf['ui']->notify_admins($text,"Host Updated",$zones);

        } // send_notify


	// Accessor functions -- things only used by this class or for
	//    debugging purposes
	function _show () {
		print("Host: name -> $this->name, tc -> $this->tc, ");
		$this->ip->_show();
		$this->domain->_show();
		$this->zone->_show();
		print("end IP<br>\n");
	}

	function _debug () {
		if ( $this->debug ) {
			$this->debug = false;
		}
		else {
			$this->debug = true;
		}
	}

} // class Host

?>
